#include <stdlib.h>
#include <iostream.h>

void vec_error(const char* msg) {
	cerr<<"error (vec): "<< msg <<"\n";
	exit(-1);
}

class Vector {
  int dim;
  double *data;
public:
  Vector (int size=1, int val=0);
  Vector (const Vector& v);
  ~Vector ();
  int size();
  double& operator[](int index);
  double operator[](int index) const;
  Vector& operator-();
  Vector operator-() const;
  Vector& operator()(int c);
  Vector& operator()(Vector& v);
  double operator*(Vector& v);
  Vector& operator= (const Vector& v);
  void print() const;
};

Vector::Vector (int size, int val) : dim (size) {
    cout<<"constructing a Vector at "<<this<<"\n";
    data = new double[size];
    for (int i=0; i<size; i++) data[i]=val;
}

Vector::Vector (const Vector& v) {
    cout<<"constructing (copying) a Vector at "<<this<<"\n";
    dim  = v.dim;
    data = new double[dim];
    for (int i=0; i<dim; i++) data[i]=v.data[i];
}

Vector:: ~Vector () {
    cout<<"destructing a Vector at "<<this<<"\n";
    delete [] data;
}

int Vector::size() { return dim;}

double& Vector::operator[](int index) {
    cout<<"operator[](int)\n";
    if ((0 <= index) && (index < dim)) return data[index];
    else vec_error("bad index");  
}

double Vector::operator[](int index) const {
    cout<<"operator[](int) const\n";
    if ((0 <= index) && (index < dim)) return data[index];
    else vec_error("bad index");  
}

Vector& Vector::operator-() {
    cout<<"operator-()\n";
    for (int i=0; i<dim; i++) data[i]=-data[i];
    return *this;
}

Vector Vector::operator-() const{
    cout<<"operator-() const\n";
	Vector v(*this);
    for (int i=0; i<dim; i++) v.data[i]=-v.data[i];
    return v;
}

Vector& Vector::operator()(int c) {
    cout<<"operator()(int)\n";
    for (int i=0; i<dim; i++) data[i]=c;
    return *this;
}

Vector& Vector::operator()(Vector& v) {
    cout<<"operator()(Vector&)\n";
    if (dim==v.dim)
      for (int i=0; i<dim; i++) data[i]=v.data[i];
    else vec_error("bad copy");
    return *this;
}

double Vector::operator*(Vector& v) {
  // Skalarprodukt
    if(dim==v.dim)
    {
       double sum=0;
       for (int i=0; i<dim; i++) sum+=data[i]*v.data[i];
       return sum;
    }
    else
    {
       vec_error("bad dimension");
       return 0;
    }
}

Vector& Vector::operator= (const Vector& v) {
    int i;
    cout<<"assigning a Vector\n";
    if (&v==this) return *this; // !!!
    if (dim==v.dim)
      for (i=0; i<dim; i++) data[i]=v.data[i];
    else vec_error("bad assignment");
    return *this;
}

void Vector::print() const {
    cout << "[" << data[0];
    for (int i=1; i<dim; i++) cout << "," << data[i];
    cout << "]\n";
}


int main() {
// initialization

   Vector v1(10);     // constructing a Vector
   Vector v2=10;      // constructing a Vector
   Vector v3=v1;      // copying a Vector
   Vector v4(v3);     // copying a Vector

// assignments

   v1(10);            // operator ()(int)
   v2=10;             // constructing a Vector, assigning a Vector
   v3=v1;             // assigning a Vector
   v4(v3);            // operator()(Vector&)

   v1.print();
   v2.print();
   v3.print();
   v4.print();

   for (int i=0; i< 10; i++) v1[i] = i;
   v4=v3=v2=v1;

   v1.print();
   v2.print();
   cout<<"v1*v2="<<v1*v2<<"\n";
   v3.print();
   v4.print();
   -v4;
   v4.print();
	
   const Vector v5(10, 11);
   cout<<v5[4]<<endl;
   Vector v6(10);
   
   v6=-v5;
   v5.print();
   v6.print();

   v1[10]=1;
// error (vec): bad index 
}
